"""
*@author: luliang
*@time: 2020-11-18 09:23:12
*@desc: 杭州交警平台驾驶人和违法数据爬取,每日4点爬取新增数据
"""


import requests

import datetime
import time
import json
from hdfs.client import Client
import smtplib
from email.mime.text import MIMEText

from beautifulsoup.parameters import parameters
from beautifulsoup.html_analyze import html_tools
from dateutil import relativedelta
from dateutil import rrule


#爬取驾驶人查询
def get_drivers_info(company_name,parameter):

    if parameter.session == None:
        return

    print('【.........开始爬取驾驶人信息.........】')
    pageNum = 1  # 记录当前爬取第几页
    driver_result = []
    total_pages = 0 #记录要爬取的总页数
    while True :
        print('【爬取第{}页数据】'.format(str(pageNum)))
        if pageNum == 1: #爬取第一页
            html_doc = requests.post(parameter.driver_url,data=parameter.create_one_page_parameter(),  headers=parameter.create_one_page_header()).content
            total_pages = html_tool.total_page(html_doc)
        else:
            html_doc = requests.post(parameter.driver_url,data=parameter.create_another_page_parameter(pageNum), headers=parameter.create_another_page_header()).content
        #获取每一页的驾驶人ID

        # print('【pages】 = ',pages)
        ids = html_tool.get_driver_id(html_doc)
        #解析当前的所有id，返回所要的结果
        id_urls = [parameter.create_id_url(id) for id in ids]
        # print(id_urls)
        for url in id_urls:
            # time.sleep(1)
            res = html_tool.retry(url,parameter)
            driver_result.append(res)
            # print(res)
        if pageNum == total_pages:
            break
        pageNum += 1

    return write_to_hdfs_partiton(driver_result, 'driver_basic_info',company_name)

    # df = pd.DataFrame(driver_result,columns=parameter.driver_column)
    # df.to_excel(parameter.driver_excel,index=None)

#爬取驾驶人违法信息
def get_driver_vio(company_name,parameter):
    #获取要查询的时间
    print('【.........开始爬取驾驶人违法信息.........】')
    now = datetime.datetime.now().date()
    start = datetime.datetime(2016, month=1, day=1).date()
    end = datetime.datetime(now.year, now.month, 1).date()

    month_count = rrule.rrule(rrule.MONTHLY, dtstart=start, until=end).count()
    days = [(start + relativedelta.relativedelta(months=i)).strftime('%Y-%m-%d') for i in range(month_count)]
    vio_results = []
    # days = ['2019-09-01']
    for day in days:
        print('【爬取驾驶人{}的违法数据】'.format(day))
        response = requests.post(parameter.create_vio_url(),data=parameter.create_vio_parameter(day,company_name),headers = parameter.create_vio_header(day)).content
        response = str(response)
        start = response.find('[')
        end = response.find(']') + 1
        # print(response[start:end])
        json_str = response[start:end].replace("dwmc","\"dwmc\"").replace("clxx","\"clxx\"").replace("jszh","\"jszh\"").replace("wfsj","\"wfsj\"").replace("xm","\"xm\"").replace("wfnr","\"wfnr\"")
        if json_str != "":
            datas = json.loads(json_str)
            for data_dict in datas:
                # print(data_dict)
                vio_results.append([v.encode('utf-8').decode('unicode_escape') for v in list(data_dict.values())])
    # print('--------------vio_results:',str(len(vio_results)))
    return write_to_hdfs_partiton(vio_results,'lawbreaking_data',company_name)
    # print(vio_results)


def write_to_hdfs_partiton(datas,table,company_name):
    try:
        client = Client("http://10.22.17.21:50070", root='/')
    except Exception:
        client = Client("http://10.22.17.20:50070", root='/')
    insert_time = time.strftime("%Y-%m-%d", time.localtime())
    [data.insert(0, company_name) for data in datas]
    add_data = list(filter(lambda x: ''.join(x).strip() != '', datas))
    [ad.append(insert_time) for ad in add_data]
    add_data_to_str = ['\t'.join(data) for data in add_data]
    # print(add_data_to_str[0:5])
    result = '\n'.join(add_data_to_str)
    print('【开始向HDFS写入数据】')
    client.write(hdfs_path='/apps/hive/warehouse/creditscores.db/{}/{}/{}'.format(table,insert_time,company_name), data=result,
                 append=False, encoding='utf-8')

    return len(datas)




def send_email(content,mess):
    current_day = str(datetime.datetime.now().date())
    subject = current_day + ':' + content + '(*＾-＾*)' # 邮件标题
    sender = "15330344779@163.com"  # 发送方
    receiver = "15330344779@163.com"  # 接收方
    # 授权码，不是邮箱密码
    password = "ISRLMHOENTBBWGSO"
    message = MIMEText(mess, "plain", "utf-8")
    # content 发送内容     "plain"文本格式   utf-8 编码格式
    message['Subject'] = subject  # 邮件标题
    message['To'] = receiver  # 收件人
    message['From'] = sender  # 发件人
    smtp = smtplib.SMTP_SSL("smtp.163.com", 994)  # 实例化smtp服务器
    smtp.login(sender, password)  # 发件人登录
    smtp.sendmail(sender, [receiver], message.as_string())
    smtp.close()



if __name__ == '__main__':

    html_tool = html_tools()
    #读取配置文件
    file = open('/root/javi/config/company.conf','r', encoding='utf-8')
    dataMat = []
    for line in file.readlines():
        dataMat.append(line.strip().split(' '))

    drivers_info = {}
    vio_info = {}
    for company_info in dataMat:
        company_name = company_info[0]

        print('【当前爬取公司：{}】'.format(company_name))
        username = company_info[1]
        password = company_info[2]
        parameter = parameters(username,password)
        drivers_add_count = get_drivers_info(company_name,parameter)
        drivers_info[company_name] = drivers_add_count
        vio_add_count = get_driver_vio(company_name,parameter)
        vio_info[company_name] = vio_add_count

    print('【数据爬取完成(*^__^*)】')
    info_str = ''
    for k,v in drivers_info.items():
        info_str += '{}今日新增驾驶人数据：{}\n'.format(k,str(v))

    for k,v in vio_info.items():
        info_str += '{}今日新增违法数据：{}\n'.format(k, str(v))
    send_email('爬取数据情况', info_str)









